package rest.jbpm.services;

import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MultivaluedMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import rest.jbpm.JbpmService;
import rest.jbpm.wrappers.Execution;
import rest.jbpm.wrappers.lists.ExecutionList;
import rest.jbpm.wrappers.lists.StringList;

@Path("exec")
public class ExecutionService  extends JbpmService  {

	private static Logger log = LoggerFactory.getLogger(ExecutionService.class);
	
	@GET
	@Produces("text/xml")
	@Path("execution/{id}")
	public String getExecution(@PathParam("id") String id) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new Execution(processEngine.getExecutionService().findExecutionById(id)), sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}	
	
	@GET
	@Produces("text/xml")
	@Path("execution/{executionId}/variableNames")
	public String getVariableNames(@PathParam("executionId") String executionId) {
		
		StringWriter sw = new StringWriter();
		try {
			log.info("variable names " + processEngine.getExecutionService().getVariableNames(executionId).toArray());
			marshaller.marshal(new StringList(processEngine.getExecutionService().getVariableNames(executionId)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
	 
	@GET
	@Produces("text/xml")
	@Path("execution/{executionId}/variable/{variableName}")
	public String getVariable(@PathParam("executionId") String executionId, @PathParam("variableName") String variableName) {
		

		return processEngine.getExecutionService().getVariable(executionId, variableName).toString();
	}
	
	@POST
	@Consumes("application/x-www-form-urlencoded")
	@Path("execution/{executionId}/variables")
	public void setVariables(MultivaluedMap<String, String> formParams,@PathParam("executionId") String executionId) {

		log.info("updating process variables for " + executionId + " with vars: ");
		Map<String,Object> vars = new HashMap<String, Object>();
		
		for(String key : formParams.keySet()) {
			
			Object value = formParams.get(key);
			log.info("name: " + key + " value: " + value);
			vars.put(key, value);
			
		}
		processEngine.getExecutionService().setVariables(executionId, vars);
	}

	@GET
	@Produces("text/xml")
	@Path("process/{id}")
	public String getProcessInstance(@PathParam("id") String id) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new Execution(processEngine.getExecutionService().createProcessInstanceQuery()
							.processInstanceId(id).uniqueResult()),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
		
	@POST
	@Produces("text/xml")
	@Path("process/{processDefinitionId}/start")
	@Consumes("application/x-www-form-urlencoded")
	public String startProcess(MultivaluedMap<String, String> formParams, @PathParam("processDefinitionId") String processDefinitionId) {
		
		log.info("starting process " + processDefinitionId + " with vars: ");
		Map<String,Object> initVars = new HashMap<String, Object>();
		
		for(String key : formParams.keySet()) {
			
			Object value = formParams.get(key);
			log.info("name: " + key + " value: " + value);
			initVars.put(key, value);
			
		}
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new Execution(processEngine.getExecutionService().startProcessInstanceById(processDefinitionId,initVars)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
		
	}
	
	@GET
	@Produces("text/xml")
	@Path("process/{processDefinitionId}/start")
	public String startProcess(@PathParam("processDefinitionId") String processDefinitionId) {
		
		log.info("starting process " + processDefinitionId + " with vars: ");
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new Execution(processEngine.getExecutionService().startProcessInstanceById(processDefinitionId)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
		
	}
	
	@POST
	@Path("process/{processInstanceId}/end")
	public void endProcess(@PathParam("processInstanceId") String processInstanceId, @FormParam("state") String state) {
		
		processEngine.getExecutionService().endProcessInstance(processInstanceId, state);
		
	}
	
	@DELETE
	@Path("process/{processInstanceId}")
	public void deleteProcess(@PathParam("processInstanceId") String processInstanceId) {
		
		processEngine.getExecutionService().deleteProcessInstance(processInstanceId);
		
	}
	
	@GET
	@Path("processes")
	@Produces("text/xml")
	public String getProcesses() {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new ExecutionList(processEngine.getExecutionService().createProcessInstanceQuery().list()), sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
		
	}
}
