package rest.jbpm.services;

import java.io.InputStream;
import java.io.StringWriter;
import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import rest.jbpm.JbpmService;
import rest.jbpm.ProcessEngineException;
import rest.jbpm.wrappers.Definition;
import rest.jbpm.wrappers.Deployment;
import rest.jbpm.wrappers.lists.DefinitionList;
import rest.jbpm.wrappers.lists.DeploymentList;

@Path("repo")
public class RepositoryService extends JbpmService {
	
	
	private static final Logger log = LoggerFactory.getLogger(RepositoryService.class);

	@GET
	@Path("/definitions")
	@Produces("text/xml")
	public String getProcessDefinitions() {
		
		if(processEngine == null)
			throw new ProcessEngineException("Fire! fire! mayday! mayday! we're going down!.. JBPM process engine is down!");
		
		
		StringWriter sw = new StringWriter();
		try {
				marshaller.marshal(
						new DefinitionList(processEngine.getRepositoryService().createProcessDefinitionQuery().list()),
						sw);
		} catch (Exception e) {
			throw new WebApplicationException(e, 500);
		}
		return sw.toString();
	}
	
	@GET
	@Path("/definition/{id}")
	@Produces("text/xml")
	public String getProcessDefinition(@PathParam("id") String id) {
		
		if(processEngine == null)
			throw new ProcessEngineException("Fire! fire! mayday! mayday! we're going down!.. JBPM process engine is down!");
		
		StringWriter sw = new StringWriter();
		try {

			marshaller.marshal(
					new Definition(processEngine.getRepositoryService().createProcessDefinitionQuery().processDefinitionId(id).uniqueResult()),
					sw);
		} catch (Exception e) {
			throw new WebApplicationException(e, 500);
		}
		return sw.toString();
	}

	@GET
	@Path("/deployments")
	@Produces("text/xml")
	public String getDeployments() {
		
		if(processEngine == null)
			throw new ProcessEngineException("Fire! fire! mayday! mayday! we're going down!.. JBPM process engine is down!");
		
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(
					new DeploymentList(processEngine.getRepositoryService().createDeploymentQuery().list()),
					sw);
		} catch (Exception e) {
			throw new WebApplicationException(e, 500);
		}
		return sw.toString();

	}
	
	@POST
	@Path("/deploy")
	@Consumes("multipart/form-data")
	@Produces("text/*")
	public String deploy(@FormParam("file") InputStream file, @FormParam("name") String name) {
		
		if(processEngine == null)
			throw new ProcessEngineException("Fire! fire! mayday! mayday! we're going down!.. JBPM process engine is down!");

		return processEngine.getRepositoryService().createDeployment().addResourceFromInputStream(name, file).deploy();
	}
	

	@GET
	@Path("/deployment/{id}")
	@Produces("text/xml")
	public String getDeployment(@PathParam("id") String id) {
		
		if(processEngine == null)
			throw new ProcessEngineException("Fire! fire! mayday! mayday! we're going down!.. JBPM process engine is down!");
		
		
		StringWriter sw = new StringWriter();
		
		try {
			marshaller.marshal(
					new Deployment(processEngine.getRepositoryService().createDeploymentQuery().deploymentId(id).uniqueResult()),
					sw);
		} catch (Exception e) {
			throw new WebApplicationException(e, 500);
		}
		return sw.toString();
	}
}
