package rest.jbpm.services;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MultivaluedMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import rest.jbpm.JbpmService;
import rest.jbpm.wrappers.Task;
import rest.jbpm.wrappers.lists.CommentList;
import rest.jbpm.wrappers.lists.TaskList;

//@Path("task")
public class TaskService extends JbpmService {

	private static Logger log = LoggerFactory.getLogger(TaskService.class);

	@GET 
	@Path("/task/{taskId}")
	@Produces("text/xml")
	public String getTask(@PathParam("taskId") String taskId) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new Task(processEngine.getTaskService().getTask(taskId)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}

	@GET
	@Path("/tasks/personal/{userId}")
	@Produces("text/xml")
	public String getPersonalTasks(@PathParam("userId") String userId) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new TaskList(processEngine.getTaskService().findPersonalTasks(userId)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
	
	@GET
	@Path("/tasks/group/{groupId}")
	@Produces("text/xml")
	public String getGroupTasks(@PathParam("groupId") String groupId) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new TaskList(processEngine.getTaskService().findGroupTasks(groupId)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
	
	@GET
	@Path("/task/{taskId}/outcomes")
	@Produces("text/xml")
	public String getOutcomes(@PathParam("taskId") String taskId) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(processEngine.getTaskService().getOutcomes(taskId),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
	
	@GET
	@Path("/task/{taskId}/variableNames")
	@Produces("text/xml")
	public String getVariableNames(@PathParam("taskId") String taskId) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(processEngine.getTaskService().getVariableNames(taskId),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
	
	@GET
	@Path("/task/{taskId}/variable/{variableName}")
	@Produces("text/xml")
	public String getVariable(@PathParam("taskId") String taskId, @PathParam("variableName") String variableName) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(processEngine.getTaskService().getVariable(taskId, variableName),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}
	
	@POST
	@Path("/task/{taskId}/variables")
	@Consumes("application/x-www-form-urlencoded")
	public void setVariables(@PathParam("taskId") String taskId, MultivaluedMap<String, String> params) {
		
		Map<String, Object> variables = new HashMap<String,Object>();
		
		for(String name : params.keySet()) {
			variables.put(name, params.getFirst(name));
		}
		
		processEngine.getTaskService().setVariables(taskId, variables);

	}
	
	@GET
	@Path("/task/{taskId}/comments")
	@Produces("text/xml")
	public String getComments(@PathParam("taskId") String taskId) {
		
		StringWriter sw = new StringWriter();
		try {
			marshaller.marshal(new CommentList(processEngine.getTaskService().getTaskComments(taskId)),sw);
		} catch (Exception e) {
			throw new WebApplicationException(e,500);
		}
		return sw.toString();
	}

	@POST
	@Path("/task/{taskId}/end/{outcome}")
	@Consumes("application/x-www-form-urlencoded")
	public void endTask(@PathParam("taskId") String taskId, 
			@PathParam("outcome") String outcome, MultivaluedMap<String, String> params) {
		
		if(outcome == null) {
			processEngine.getTaskService().completeTask(taskId);
			return;
		}
		
		if(params != null) {
			
			Map<String, Object> variables = new HashMap<String,Object>();
			
			for(String name : params.keySet()) {
				variables.put(name, params.getFirst(name));
			}
			
			processEngine.getTaskService().completeTask(taskId,outcome,variables);
			return;
		}
		
		processEngine.getTaskService().completeTask(taskId, outcome);

	}
	
	
}
